<%inherit file="base.html"/>
<%namespace file="site_helpers.mako" name="helpers"/>

<h1>${helpers.render_translation("primary_section_recent_notes_heading", {})}</h1>
${helpers.render_recent_notes()}
