<%namespace file="site_helpers.mako" name="helpers"/>
<div class="f">
  <div class="f-i">
    <div class="f-a">
      ${helpers.render_chunk("footer_copyrights")}
    </div>
    <div class="f-s">
      ${helpers.render_network_list(site, class_="f-nw")}
    </div>
  </div>
</div>
