<%namespace file="site_helpers.mako" name="helpers"/>
<!doctype html>
<html lang="${helpers.render_site_language(site)}">
  <head>
    ${helpers.render_meta()}
    <title>${helpers.render_title(breadcrumbs)}</title>
    ${helpers.render_styles()}
    ${helpers.render_feeds()}
  </head>
  <body>
    <%include file="header.html"/>
    <div class="c">
      <div class="c-i">
        <div class="c-p">${next.body()}</div>
        <div class="c-s">
          <%block name="secondary_in_start"/>
          ${helpers.render_secondary_about()}
          ${helpers.render_secondary_recent_notes()}
          ${helpers.render_secondary_popular_tags()}
          <%block name="secondary_in_end"/>
        </div>
      </div>
    </div>
    <%include file="footer.html"/>
    ${helpers.render_scripts()}
  </body>
</html>
